package de.fahsel.dewis;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * Diese abstrakte Klasse repräsentiert eine ladbare Organisation. 
 * 
 * @author Peter Fahsel
 *
 */
public abstract class LoadableOrganisation extends AbstractOrganisation
{
	private static final String CSV_HEADLINE = "id|nachname|vorname|titel|verein|mglnr|status|dwz|dwzindex|turniercode|turnierende|fideid|fideelo|fidetitel";
	
	private List<Player> players;
	
	protected LoadableOrganisation(String zps)
	{
		super(zps);
	}

	protected abstract String getLoadUrl();
	
	/**
	 * Lädt die Spielerliste dieses Vereins / Verbands
	 * 
	 * @return Eine Liste von {@link Player}-Objekten
	 * @throws IOException
	 */
	public List<Player> getPlayers() throws IOException
	{
		if (players == null)
		{
			loadPlayers();
		}
		return players;
	}
	
	private void loadPlayers() throws IOException
	{
		players = new ArrayList<>();
		
		URL               url = new URL(getLoadUrl().replace("$$zps$$", getZps().toString()));
		HttpURLConnection con = (HttpURLConnection) url.openConnection();
		String            inputLine;
		
		try (BufferedReader r = new BufferedReader(new InputStreamReader(con.getInputStream()))) {
			while ((inputLine = r.readLine()) != null)
			{
				if (!inputLine.startsWith(CSV_HEADLINE))
				{
					String[] playerSplit = inputLine.split("\\|", -1);
					
					if (playerSplit.length < 14)
					{
						continue;
					}
					
					Player player = Player.getInstance(playerSplit[0]);
					
					player.setLastName(playerSplit[1]);
					player.setFirstName(playerSplit[2]);
					player.setTitle(playerSplit[3]);
					player.setDwz(playerSplit[7]);
					player.setDwzIndex(playerSplit[8]);
					player.setLastTournamentCode(playerSplit[9]);
					player.setLastTournamentEnd(playerSplit[10]);
					player.setFideId(playerSplit[11]);
					player.setFideElo(playerSplit[12]);
					player.setFideTitle(playerSplit[13]);
					
					ClubMembership membership = new ClubMembership(Club.getInstance(playerSplit[4]), playerSplit[6], playerSplit[5]);
					player.addOrReplaceMembership(membership);
					
					addPlayer(player);
				}
			}
		}
	}
	
	private void addPlayer(Player player)
	{
		players.add(player);
	}

}
